@@ -23,6 +23,7 @@
  * Original code for IsoLatin1 and UTF-16 by "Martin J. Duerst" <duerst@w3.org>
  */
 
+#define IN_LIBXML
 #include "libxml.h"
 
 #include <string.h>
@@ -38,8 +39,8 @@
 #include <errno.h>
 #endif
 #endif
-#include <libxml/encoding.h>
 #include <libxml/xmlmemory.h>
+#include <libxml/encoding.h>
 #ifdef LIBXML_HTML_ENABLED
 #include <libxml/HTMLparser.h>
 #endif
@@ -199,7 +200,7 @@ error:
  * null-terminated. This function is not super-strict, as it will
  * allow longer utf-8 sequences than necessary. Note that Java is
  * capable of producing these sequences if provoked. Also note, this
- * routine checks for the 4-byte maxiumum size, but does not check for
+ * routine checks for the 4-byte maximum size, but does not check for
  * 0x10ffff maximum value.
  *
  * Return value: true if @utf is valid.
@@ -417,7 +418,7 @@ xmlUTF8Strsub(const xmlChar *utf, int start, int len) {
  * block of chars out.
  * Returns 0 if success, or -1 otherwise
  * The value of @inlen after return is the number of octets consumed
- *     as the return value is positive, else unpredictiable.
+ *     as the return value is positive, else unpredictable.
  * The value of @outlen after return is the number of ocetes consumed.
  */
 static int
@@ -469,7 +470,7 @@ asciiToUTF8(unsigned char* out, int *outlen,
  *
  * Returns 0 if success, -2 if the transcoding fails, or -1 otherwise
  * The value of @inlen after return is the number of octets consumed
- *     as the return value is positive, else unpredictiable.
+ *     as the return value is positive, else unpredictable.
  * The value of @outlen after return is the number of ocetes consumed.
  */
 static int
@@ -551,7 +552,7 @@ UTF8Toascii(unsigned char* out, int *outlen,
  * block of chars out.
  * Returns 0 if success, or -1 otherwise
  * The value of @inlen after return is the number of octets consumed
- *     as the return value is positive, else unpredictiable.
+ *     as the return value is positive, else unpredictable.
  * The value of @outlen after return is the number of ocetes consumed.
  */
 int
@@ -559,35 +560,38 @@ isolat1ToUTF8(unsigned char* out, int *outlen,
               const unsigned char* in, int *inlen) {
     unsigned char* outstart = out;
     const unsigned char* base = in;
-    const unsigned char* processed = in;
     unsigned char* outend = out + *outlen;
     const unsigned char* inend;
-    unsigned int c;
+    const unsigned char* instop;
+    xmlChar c = *in;
 
     inend = in + (*inlen);
-    while (in < inend) {
-	c = *in++;
-
-        if (out >= outend)
-	    break;
-
-        if (c < 0x80) {
-	    *out++ =  c;
-	    processed++;
-	    continue;
-	} else {
+    instop = inend;
+    
+    while (in < inend && out < outend - 1) {
+    	if (c >= 0x80) {
 	    *out++= ((c >>  6) & 0x1F) | 0xC0;
-            if (out >= outend)
-		break;
             *out++= (c & 0x3F) | 0x80;
-	    processed++;
-        }
+	    ++in;
+	    c = *in;
+	}
+	if (instop - in > outend - out) instop = in + (outend - out); 
+	while (c < 0x80 && in < instop) {
+	    *out++ =  c;
+	    ++in;
+	    c = *in;
+	}
+    }	
+    if (in < inend && out < outend && c < 0x80) {
+        *out++ =  c;
+	++in;
     }
     *outlen = out - outstart;
-    *inlen = processed - base;
+    *inlen = in - base;
     return(0);
 }
 
+
 /**
  * UTF8Toisolat1:
  * @out:  a pointer to an array of bytes to store the result
@@ -600,7 +604,7 @@ isolat1ToUTF8(unsigned char* out, int *outlen,
  *
  * Returns 0 if success, -2 if the transcoding fails, or -1 otherwise
  * The value of @inlen after return is the number of octets consumed
- *     as the return value is positive, else unpredictiable.
+ *     as the return value is positive, else unpredictable.
  * The value of @outlen after return is the number of ocetes consumed.
  */
 int
@@ -684,14 +688,14 @@ UTF8Toisolat1(unsigned char* out, int *outlen,
  * @inlenb:  the length of @in in UTF-16LE chars
  *
  * Take a block of UTF-16LE ushorts in and try to convert it to an UTF-8
- * block of chars out. This function assume the endian properity
+ * block of chars out. This function assume the endian property
  * is the same between the native type of this machine and the
  * inputed one.
  *
  * Returns the number of byte written, or -1 by lack of space, or -2
  *     if the transcoding fails (for *in is not valid utf16 string)
  *     The value of *inlen after return is the number of octets consumed
- *     as the return value is positive, else unpredictiable.
+ *     as the return value is positive, else unpredictable.
  */
 static int
 UTF16LEToUTF8(unsigned char* out, int *outlen,
@@ -891,14 +895,14 @@ UTF8ToUTF16LE(unsigned char* outb, int *outlen,
  * @inlenb:  the length of @in in UTF-16 chars
  *
  * Take a block of UTF-16 ushorts in and try to convert it to an UTF-8
- * block of chars out. This function assume the endian properity
+ * block of chars out. This function assume the endian property
  * is the same between the native type of this machine and the
  * inputed one.
  *
  * Returns the number of byte written, or -1 by lack of space, or -2
  *     if the transcoding fails (for *in is not valid utf16 string)
  * The value of *inlen after return is the number of octets consumed
- *     as the return value is positive, else unpredictiable.
+ *     as the return value is positive, else unpredictable.
  */
 static int
 UTF16BEToUTF8(unsigned char* out, int *outlen,
@@ -1170,6 +1174,7 @@ xmlCleanupEncodingAliases(void) {
     xmlCharEncodingAliasesNb = 0;
     xmlCharEncodingAliasesMax = 0;
     xmlFree(xmlCharEncodingAliases);
+    xmlCharEncodingAliases = NULL;
 }
 
 /**
@@ -1304,7 +1309,7 @@ xmlDelEncodingAlias(const char *alias) {
  * xmlParseCharEncoding:
  * @name:  the encoding name as parsed, in UTF-8 format (ASCII actually)
  *
- * Conpare the string to the known encoding schemes already known. Note
+ * Compare the string to the known encoding schemes already known. Note
  * that the comparison is case insensitive accordingly to the section
  * [XML] 4.3.3 Character Encoding in Entities.
  * 
@@ -1476,9 +1481,10 @@ static xmlCharEncodingHandlerPtr xmlDefaultCharEncodingHandler = NULL;
  * @output:  the xmlCharEncodingOutputFunc to write that encoding
  *
  * Create and registers an xmlCharEncodingHandler.
+ *
  * Returns the xmlCharEncodingHandlerPtr created (or NULL in case of error).
  */
-static xmlCharEncodingHandlerPtr
+xmlCharEncodingHandlerPtr
 xmlNewCharEncodingHandler(const char *name, 
                           xmlCharEncodingInputFunc input,
                           xmlCharEncodingOutputFunc output) {
@@ -1616,7 +1622,7 @@ xmlCleanupCharEncodingHandlers(void) {
  * xmlRegisterCharEncodingHandler:
  * @handler:  the xmlCharEncodingHandlerPtr handler block
  *
- * Register the char encoding handler, surprizing, isn't it ?
+ * Register the char encoding handler, surprising, isn't it ?
  */
 void
 xmlRegisterCharEncodingHandler(xmlCharEncodingHandlerPtr handler) {
@@ -1641,7 +1647,7 @@ xmlRegisterCharEncodingHandler(xmlCharEncodingHandlerPtr handler) {
  * xmlGetCharEncodingHandler:
  * @enc:  an xmlCharEncoding value.
  *
- * Search in the registrered set the handler able to read/write that encoding.
+ * Search in the registered set the handler able to read/write that encoding.
  *
  * Returns the handler or NULL if not found
  */
@@ -1771,7 +1777,7 @@ xmlGetCharEncodingHandler(xmlCharEncoding enc) {
  * xmlFindCharEncodingHandler:
  * @name:  a string describing the char encoding.
  *
- * Search in the registrered set the handler able to read/write that encoding.
+ * Search in the registered set the handler able to read/write that encoding.
  *
  * Returns the handler or NULL if not found
  */
@@ -1887,7 +1893,7 @@ xmlFindCharEncodingHandler(const char *name) {
  *     -3 if there the last byte can't form a single output char.
  *     
  * The value of @inlen after return is the number of octets consumed
- *     as the return value is positive, else unpredictiable.
+ *     as the return value is positive, else unpredictable.
  * The value of @outlen after return is the number of ocetes consumed.
  */
 static int
@@ -1973,7 +1979,7 @@ xmlCharEncFirstLine(xmlCharEncodingHandler *handler, xmlBufferPtr out,
     /*
      * echo '<?xml version="1.0" encoding="UCS4"?>' | wc -c => 38
      * 45 chars should be sufficient to reach the end of the encoding
-     * decalration without going too far inside the document content.
+     * declaration without going too far inside the document content.
      */
     written = 45;
 
@@ -2027,7 +2033,7 @@ xmlCharEncFirstLine(xmlCharEncodingHandler *handler, xmlBufferPtr out,
 
 /**
  * xmlCharEncInFunc:
- * @handler:	char enconding transformation data structure
+ * @handler:	char encoding transformation data structure
  * @out:  an xmlBuffer for the output.
  * @in:  an xmlBuffer for the input
  *     
@@ -2178,7 +2184,7 @@ retry:
     }
 
     /*
-     * Convertion itself.
+     * Conversion itself.
      */
     toconv = in->use;
     if (toconv == 0)
@@ -2265,7 +2271,7 @@ retry:
 		 * and continue the transcoding phase, hoping the error
 		 * did not mangle the encoder state.
 		 */
-		sprintf((char *) charref, "&#%d;", cur);
+		snprintf((char *) charref, sizeof(charref), "&#%d;", cur);
 		xmlBufferShrink(in, len);
 		xmlBufferAddHead(in, charref, -1);
 
@@ -2289,7 +2295,7 @@ retry:
  * xmlCharEncCloseFunc:
  * @handler:	char enconding transformation data structure
  *     
- * Generic front-end for hencoding handler close function
+ * Generic front-end for encoding handler close function
  *
  * Returns 0 if success, or -1 in case of error
  */
@@ -2300,7 +2306,7 @@ xmlCharEncCloseFunc(xmlCharEncodingHandler *handler) {
     if (handler->name == NULL) return(-1);
 #ifdef LIBXML_ICONV_ENABLED
     /*
-     * Iconv handlers can be oused only once, free the whole block.
+     * Iconv handlers can be used only once, free the whole block.
      * and the associated icon resources.
      */
     if ((handler->iconv_out != NULL) || (handler->iconv_in != NULL)) {
