@@ -74,7 +74,7 @@ class SlowParser(xmllib.XMLParser):
 def getparser(target = None):
     # get the fastest available parser, and attach it to an
     # unmarshalling object.  return both objects.
-    if target == None:
+    if target is None:
         target = docParser()
     if FastParser:
         return FastParser(target), target
@@ -350,7 +350,7 @@ def print_function_wrapper(name, output, export, include):
 	    if args[1][1] == "char *" or args[1][1] == "xmlChar *":
 		c_call = "\n    if (%s->%s != NULL) xmlFree(%s->%s);\n" % (
 		                 args[0][0], args[1][0], args[0][0], args[1][0])
-		c_call = c_call + "    %s->%s = xmlStrdup(%s);\n" % (args[0][0],
+		c_call = c_call + "    %s->%s = xmlStrdup((const xmlChar *)%s);\n" % (args[0][0],
 		                 args[1][0], args[1][0])
 	    else:
 		c_call = "\n    %s->%s = %s;\n" % (args[0][0], args[1][0],
@@ -383,21 +383,49 @@ def print_function_wrapper(name, output, export, include):
             unknown_types[ret[0]] = [name]
         return -1
 
-    include.write("PyObject * ")
-    include.write("libxml_%s(PyObject *self, PyObject *args);\n" % (name))
+    if file == "debugXML":
+        include.write("#ifdef LIBXML_DEBUG_ENABLED\n");
+        export.write("#ifdef LIBXML_DEBUG_ENABLED\n");
+        output.write("#ifdef LIBXML_DEBUG_ENABLED\n");
+    elif file == "HTMLtree" or file == "HTMLparser":
+        include.write("#ifdef LIBXML_HTML_ENABLED\n");
+        export.write("#ifdef LIBXML_HTML_ENABLED\n");
+        output.write("#ifdef LIBXML_HTML_ENABLED\n");
+    elif file == "c14n":
+        include.write("#ifdef LIBXML_C14N_ENABLED\n");
+        export.write("#ifdef LIBXML_C14N_ENABLED\n");
+        output.write("#ifdef LIBXML_C14N_ENABLED\n");
+    elif file == "xpathInternals" or file == "xpath":
+        include.write("#ifdef LIBXML_XPATH_ENABLED\n");
+        export.write("#ifdef LIBXML_XPATH_ENABLED\n");
+        output.write("#ifdef LIBXML_XPATH_ENABLED\n");
+    elif file == "xpointer":
+        include.write("#ifdef LIBXML_XPTR_ENABLED\n");
+        export.write("#ifdef LIBXML_XPTR_ENABLED\n");
+        output.write("#ifdef LIBXML_XPTR_ENABLED\n");
+    elif file == "xinclude":
+        include.write("#ifdef LIBXML_XINCLUDE_ENABLED\n");
+        export.write("#ifdef LIBXML_XINCLUDE_ENABLED\n");
+        output.write("#ifdef LIBXML_XINCLUDE_ENABLED\n");
 
-    export.write("    { \"%s\", libxml_%s, METH_VARARGS, NULL },\n" %
+    include.write("PyObject * ")
+    include.write("libxml_%s(PyObject *self, PyObject *args);\n" % (name));
+
+    export.write("    { (char *)\"%s\", libxml_%s, METH_VARARGS, NULL },\n" %
                  (name, name))
 
     if file == "python":
         # Those have been manually generated
         return 1
-    if file == "python_accessor" and ret[0] != "void" and ret[2] == None:
+    if file == "python_accessor" and ret[0] != "void" and ret[2] is None:
         # Those have been manually generated
         return 1
 
     output.write("PyObject *\n")
-    output.write("libxml_%s(PyObject *self, PyObject *args) {\n" % (name))
+    output.write("libxml_%s(ATTRIBUTE_UNUSED PyObject *self," % (name))
+    if format == "":
+	output.write("ATTRIBUTE_UNUSED ")
+    output.write(" PyObject *args) {\n")
     if ret[0] != 'void':
         output.write("    PyObject *py_retval;\n")
     if c_return != "":
@@ -405,7 +433,7 @@ def print_function_wrapper(name, output, export, include):
     if c_args != "":
         output.write(c_args)
     if format != "":
-        output.write("\n    if (!PyArg_ParseTuple(args, \"%s\"%s))\n" %
+        output.write("\n    if (!PyArg_ParseTuple(args, (char *)\"%s\"%s))\n" %
                      (format, format_args))
         output.write("        return(NULL);\n")
     if c_convert != "":
@@ -414,6 +442,30 @@ def print_function_wrapper(name, output, export, include):
     output.write(c_call)
     output.write(ret_convert)
     output.write("}\n\n")
+    if file == "debugXML":
+        include.write("#endif /* LIBXML_DEBUG_ENABLED */\n");
+        export.write("#endif /* LIBXML_DEBUG_ENABLED */\n");
+        output.write("#endif /* LIBXML_DEBUG_ENABLED */\n");
+    elif file == "HTMLtree" or file == "HTMLparser":
+        include.write("#endif /* LIBXML_HTML_ENABLED */\n");
+        export.write("#endif /* LIBXML_HTML_ENABLED */\n");
+        output.write("#endif /* LIBXML_HTML_ENABLED */\n");
+    elif file == "c14n":
+        include.write("#endif /* LIBXML_C14N_ENABLED */\n");
+        export.write("#endif /* LIBXML_C14N_ENABLED */\n");
+        output.write("#endif /* LIBXML_C14N_ENABLED */\n");
+    elif file == "xpathInternals" or file == "xpath":
+        include.write("#endif /* LIBXML_XPATH_ENABLED */\n");
+        export.write("#endif /* LIBXML_XPATH_ENABLED */\n");
+        output.write("#endif /* LIBXML_XPATH_ENABLED */\n");
+    elif file == "xpointer":
+        include.write("#endif /* LIBXML_XPTR_ENABLED */\n");
+        export.write("#endif /* LIBXML_XPTR_ENABLED */\n");
+        output.write("#endif /* LIBXML_XPTR_ENABLED */\n");
+    elif file == "xinclude":
+        include.write("#endif /* LIBXML_XINCLUDE_ENABLED */\n");
+        export.write("#endif /* LIBXML_XINCLUDE_ENABLED */\n");
+        output.write("#endif /* LIBXML_XINCLUDE_ENABLED */\n");
     return 1
 
 def buildStubs():
@@ -465,6 +517,8 @@ def buildStubs():
     wrapper = open("libxml2-py.c", "w")
     wrapper.write("/* Generated */\n\n")
     wrapper.write("#include <Python.h>\n")
+    wrapper.write("#include \"config.h\"\n")
+    wrapper.write("#include <libxml/xmlversion.h>\n")
     wrapper.write("#include <libxml/tree.h>\n")
     wrapper.write("#include \"libxml_wrap.h\"\n")
     wrapper.write("#include \"libxml2-py.h\"\n\n")
@@ -525,6 +579,8 @@ classes_type = {
     "xmlXPathParserContextPtr": ("._o", "xpathParserContext(_obj=%s)", "xpathParserContext"),
     "xmlParserCtxtPtr": ("._o", "parserCtxt(_obj=%s)", "parserCtxt"),
     "xmlParserCtxt *": ("._o", "parserCtxt(_obj=%s)", "parserCtxt"),
+    "htmlParserCtxtPtr": ("._o", "parserCtxt(_obj=%s)", "parserCtxt"),
+    "htmlParserCtxt *": ("._o", "parserCtxt(_obj=%s)", "parserCtxt"),
     "xmlCatalogPtr": ("._o", "catalog(_obj=%s)", "catalog"),
     "xmlURIPtr": ("._o", "URI(_obj=%s)", "URI"),
 }
@@ -644,7 +700,7 @@ def functionCompare(info1, info2):
     return 0
 
 def writeDoc(name, args, indent, output):
-     if functions[name][0] == None or functions[name][0] == "":
+     if functions[name][0] is None or functions[name][0] == "":
          return
      val = functions[name][0]
      val = string.replace(val, "NULL", "None");
@@ -782,7 +838,7 @@ def buildWrappers():
 
 	    for arg in args:
 		if classes_type.has_key(arg[1]):
-		    classes.write("    if %s == None: %s__o = None\n" %
+		    classes.write("    if %s is None: %s__o = None\n" %
 				  (arg[0], arg[0]))
 		    classes.write("    else: %s__o = %s%s\n" %
 				  (arg[0], arg[0], classes_type[arg[1]][0]))
@@ -806,22 +862,22 @@ def buildWrappers():
 		    # Raise an exception
 		    #
 		    if functions_noexcept.has_key(name):
-		        classes.write("    if ret == None:return None\n");
+		        classes.write("    if ret is None:return None\n");
 		    elif string.find(name, "URI") >= 0:
 			classes.write(
-			"    if ret == None:raise uriError('%s() failed')\n"
+			"    if ret is None:raise uriError('%s() failed')\n"
 			              % (name))
 		    elif string.find(name, "XPath") >= 0:
 			classes.write(
-			"    if ret == None:raise xpathError('%s() failed')\n"
+			"    if ret is None:raise xpathError('%s() failed')\n"
 			              % (name))
 		    elif string.find(name, "Parse") >= 0:
 			classes.write(
-			"    if ret == None:raise parserError('%s() failed')\n"
+			"    if ret is None:raise parserError('%s() failed')\n"
 			              % (name))
 		    else:
 			classes.write(
-			"    if ret == None:raise treeError('%s() failed')\n"
+			"    if ret is None:raise treeError('%s() failed')\n"
 			              % (name))
 		    classes.write("    return ");
 		    classes.write(classes_type[ret[0]][1] % ("ret"));
@@ -891,7 +947,7 @@ def buildWrappers():
 		for arg in args:
 		    if classes_type.has_key(arg[1]):
 			if n != index:
-			    classes.write("        if %s == None: %s__o = None\n" %
+			    classes.write("        if %s is None: %s__o = None\n" %
 					  (arg[0], arg[0]))
 			    classes.write("        else: %s__o = %s%s\n" %
 					  (arg[0], arg[0], classes_type[arg[1]][0]))
@@ -922,22 +978,22 @@ def buildWrappers():
 			#
 			if functions_noexcept.has_key(name):
 			    classes.write(
-			        "        if ret == None:return None\n");
+			        "        if ret is None:return None\n");
 			elif string.find(name, "URI") >= 0:
 			    classes.write(
-		    "        if ret == None:raise uriError('%s() failed')\n"
+		    "        if ret is None:raise uriError('%s() failed')\n"
 					  % (name))
 			elif string.find(name, "XPath") >= 0:
 			    classes.write(
-		    "        if ret == None:raise xpathError('%s() failed')\n"
+		    "        if ret is None:raise xpathError('%s() failed')\n"
 					  % (name))
 			elif string.find(name, "Parse") >= 0:
 			    classes.write(
-		    "        if ret == None:raise parserError('%s() failed')\n"
+		    "        if ret is None:raise parserError('%s() failed')\n"
 					  % (name))
 			else:
 			    classes.write(
-		    "        if ret == None:raise treeError('%s() failed')\n"
+		    "        if ret is None:raise treeError('%s() failed')\n"
 					  % (name))
 			classes.write("        return ");
 			classes.write(classes_type[ret[0]][1] % ("ret"));
@@ -948,22 +1004,22 @@ def buildWrappers():
 			#
 			if functions_noexcept.has_key(name):
 			    classes.write(
-			        "        if ret == None:return None");
+			        "        if ret is None:return None");
 			elif string.find(name, "URI") >= 0:
 			    classes.write(
-		    "        if ret == None:raise uriError('%s() failed')\n"
+		    "        if ret is None:raise uriError('%s() failed')\n"
 					  % (name))
 			elif string.find(name, "XPath") >= 0:
 			    classes.write(
-		    "        if ret == None:raise xpathError('%s() failed')\n"
+		    "        if ret is None:raise xpathError('%s() failed')\n"
 					  % (name))
 			elif string.find(name, "Parse") >= 0:
 			    classes.write(
-		    "        if ret == None:raise parserError('%s() failed')\n"
+		    "        if ret is None:raise parserError('%s() failed')\n"
 					  % (name))
 			else:
 			    classes.write(
-		    "        if ret == None:raise treeError('%s() failed')\n"
+		    "        if ret is None:raise treeError('%s() failed')\n"
 					  % (name))
 			classes.write("        return ");
 			classes.write(converter_type[ret[0]] % ("ret"));
